<?php

namespace NoccyLabs\React\CommandBus;
use Evenement\EventEmitterInterface;
use Evenement\EventEmitterTrait;

/**
 * A collection of commands that can be executed via CommandBusInterface
 *
 */
trait CommandResolverTrait implements EventEmitterInterface
{
    use EventEmitterTrait;

    const EVENT_REGISTERED = 'registered';
    const EVENT_UNREGISTERED = 'unregistered';

    /** @var array<string,Command> */
    private array $commands = [];

    public function registerCommand(string $command, callable $handler): void
    {
        $isNew = !array_key_exists($command, $this->commands);

        $this->commands[$command] = new Command($command, $handler);

        if ($isNew) {
            $this->emit(self::EVENT_REGISTERED, [ $command ]);
        }
    }

    public function unregisterCommand(string $command): void
    {
        if (!array_key_exists($command, $this->commands)) {
            return;
        }

        unset($this->commands[$command]);

        $this->emit(self::EVENT_UNREGISTERED, [ $command ]);
    }

    public function findCommand(string $command): ?Command
    {
        return $this->commands[$command] ?? null;
    }

    public function getCommandNames(): array
    {
        return array_keys($this->commands);
    }
}

